import uuid, json

from requests import Request, Session

class Base(object):

    def __init__(self, merchant):

        self.merchant = merchant

    def send_request(self, method, uri, data=None, params=None):

        s = Session()

        body = data

        headers = {
            'User-Agent': "CieloEcommerce/3.0 Python SDK",
            'RequestId': str(uuid.uuid4()),
            'MerchantId': self.merchant.id,
            'MerchantKey': self.merchant.key
        }

        if not body:
            headers['Content-Length'] = '0'
        else:
            headers["Content-Type"] = "application/json"

            if not isinstance(data, dict):
                body = body.toJSON()

        req = Request(method, uri, data=body, headers=headers, params=params)

        prep = s.prepare_request(req)

        response = s.send(prep)

        if 'json' in response.headers['Content-Type'].lower():
            answers = response.json()
        else:
            answers = [{
                'Code': str(response.status_code),
                'Message': response.text
            }]

        if response.status_code >= 400:
            errors = []

            for answer in answers:
                errors.append('\r\n * [%s] %s\r\n' % (answer['Code'], answer['Message']))

            data_send = json.loads(body or 'null')

            raise Exception('\r\n%s\r\nMethod: %s\r\nUri: %s\r\nData: %s' % (
            ''.join(errors), method, response.url, json.dumps(data_send, indent=2)))

        return answers




from .base import Base

class CreateSale(Base):

    def __init__(self, merchant, environment):

        super(CreateSale, self).__init__(merchant)

        self.environment = environment

    def execute(self, sale):

        uri = '%s1/sales' % self.environment.api

        response = self.send_request("POST", uri, sale)

        sale.update_return(response)

        return response




from .objectJSON import ObjectJSON

PAYMENTTYPE_CREDITCARD = "CreditCard"
PAYMENTTYPE_DEBITCARD = "DebitCard"
PAYMENTTYPE_ELECTRONIC_TRANSFER = "ElectronicTransfer"
PAYMENTTYPE_BOLETO = "Boleto"

PROVIDER_BRADESCO = "Bradesco"
PROVIDER_BANCO_DO_BRASIL = "BancoDoBrasil"
PROVIDER_SIMULADO = "Simulado"

class Payment(ObjectJSON):

    def __init__(self, amount, installments = 1):

        self.amount = amount
        self.service_tax_amount = None
        self.installments = installments
        self.interest = None
        self.capture = None
        self.authenticate = None
        self.recurrent = None
        self.recurrent_payment = None
        self.credit_card = None
        self.debit_card = None
        self.authentication_url = None
        self.proof_of_sale = None
        self.authorization_code = None
        self.soft_descriptor = None
        self.return_url = None
        self.provider = None
        self.payment_id = None
        self.tid = None
        self.type = None
        self.received_date = None
        self.captured_amount = None
        self.captured_date = None
        self.currency = None
        self.country = None
        self.return_code = None
        self.return_message = None
        self.status = None
        self.links = None
        self.extra_data_collection = None
        self.expiration_date = None
        self.url = None
        self.number = None
        self.bar_code_number = None
        self.digitable_line = None
        self.address = None

        #Boleto
        self.boleto_number = None
        self.assignor = None
        self.demonstrative = None
        self.identification = None
        self.instructions = None


    def prepare(self):

        if self.credit_card:
            self.type = PAYMENTTYPE_CREDITCARD



from .base import Base

class CardBin(Base):

    def __init__(self, merchant, environment):

        super(CardBin, self).__init__(merchant)

        self.environment = environment

    def execute(self, bin):

        uri = '%s1/cardBin/%s' % (self.environment.api_query, bin)

        return self.send_request('GET', uri)



from .base import Base

class QuerySale(Base):

    def __init__(self, merchant, environment):

        super(QuerySale, self).__init__(merchant)

        self.environment = environment

    def execute(self, payment_id):

        uri = '%s1/sales/%s' % (self.environment.api_query, payment_id)

        return self.send_request("GET", uri)




from .request.createSale import CreateSale
from .request.querySale import QuerySale
from .request.cardBin import CardBin
from .request.updateSale import UpdateSale
from .request.createCardToken import CreateCardToken
from .request.queryRecorrency import QueryRecorrency
from .request.deactivateRecorrency import DeactivateRecorrency
from .request.reactivateRecorrency import ReactivateRecorrency

class CieloEcommerce(object):

    def __init__(self, merchant, environment):

        self.environment = environment
        self.merchant = merchant

    def create_sale(self, sale):

        request = CreateSale(self.merchant, self.environment)

        return request.execute(sale)

    def capture_sale(self, payment_id, amount = None, service_tax_amount = None):
        request = UpdateSale('capture', self.merchant, self.environment)

        request.amount = amount
        request.service_tax_amount = service_tax_amount

        return request.execute(payment_id)

    def cancel_sale(self, payment_id, amount = None):
        request = UpdateSale('void', self.merchant, self.environment)

        request.amount = amount

        return request.execute(payment_id)

    def get_sale(self, payment_id):
        request = QuerySale(self.merchant, self.environment)

        return request.execute(payment_id)

    def get_card_bin(self, binCard):
        request = CardBin(self.merchant, self.environment)

        return request.execute(binCard)
    
    def create_card_token(self, creditCard):

        request = CreateCardToken(self.merchant, self.environment)

        return request.execute(creditCard)

    def get_recurrent_payment(self, recurrent_payment_id):

        request = QueryRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)

    def deactivate_recurrent_payment(self, recurrent_payment_id):

        request = DeactivateRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)

    def reactivate_recurrent_payment(self, recurrent_payment_id):

        request = ReactivateRecorrency(self.merchant, self.environment)

        return request.execute(recurrent_payment_id)





from .environment import *
from .merchant import *

from .sale import *
from .customer import *
from .creditCard import *
from .debitCard import *
from .payment import *

from .recurrentPayment import *

from .cieloEcommerce import *



from setuptools import find_packages
from distutils.core import setup

def get_version():
    return open('version.txt', 'r').read().strip()

setup(
    author='Halyson Sampaio',
    author_email='halyson@lojaspompeia.com.br',
    description='Pacote customizado da sdk python da cielo, https://github.com/DeveloperCielo/API-3.0-Python',
    license='MIT',
    name='lins_cieloApi3',
    packages=find_packages(),
    url='https://bitbucket.org/grupolinsferrao/pypck-lins-cieloapi3/',
    version=get_version(),
    zip_safe=False
)



from .objectJSON import ObjectJSON

class CreditCard(ObjectJSON):

    def __init__(self, security_code, brand):

        self.card_number = None
        self.holder = None
        self.expiration_date = None
        self.security_code = security_code
        self.save_card = None
        self.brand = brand
        self.card_token = None
        self.customer_name = None


    def update_return(self, response_return):

        self.card_token = response_return['CardToken']
